@@ -23,10 +23,16 @@ There are three submission events:
    Submitted in :func:`~darc.crawl.loader` function call, after the
    loading process of the URL using :mod:`selenium`.
 
+.. seealso::
+
+   Please refer to :doc:`data schema </demo/schema>` for more
+   information about the submission data.
+
 """
 
 import base64
 import dataclasses
+import datetime
 import glob
 import json
 import os
@@ -40,14 +46,16 @@ import stem.util.term
 
 import darc.typing as typing
 from darc.const import DEBUG, PATH_DB
+from darc.db import _db_operation
 from darc.error import APIRequestFailed, DatabaseOperaionFailed, render_error
 from darc.link import Link
 from darc.model import (HostnameModel, HostsModel, RequestsHistoryModel, RequestsModel, RobotsModel,
                         SeleniumModel, SitemapModel, URLModel)
+from darc.model.utils import Proxy
 from darc.requests import null_session
 
 # type alias
-File = typing.Dict[str, typing.Union[str, typing.ByteString]]
+File = typing.Dict[str, typing.AnyStr]
 Domain = typing.Literal['new_host', 'requests', 'selenium']
 
 # save submitted data to database
@@ -72,6 +80,9 @@ if DEBUG:
     print(stem.util.term.format(f'SELENIUM: {API_SELENIUM}', stem.util.term.Color.MAGENTA))  # pylint: disable=no-member
     print(stem.util.term.format('-' * shutil.get_terminal_size().columns, stem.util.term.Color.MAGENTA))  # pylint: disable=no-member
 
+# UNIX epoch
+EPOCH = datetime.datetime(1970, 1, 1, 0, 0)  # 1970-01-01T00:00:00
+
 
 def get_metadata(link: Link) -> typing.Dict[str, str]:
     """Generate metadata field.
@@ -117,7 +128,7 @@ def get_robots(link: Link) -> typing.Optional[File]:  # pylint: disable=inconsis
     """
     path = os.path.join(link.base, 'robots.txt')
     if not os.path.isfile(path):
-        return
+        return None
     with open(path, 'rb') as file:
         content = file.read()
     data = dict(
@@ -127,7 +138,7 @@ def get_robots(link: Link) -> typing.Optional[File]:  # pylint: disable=inconsis
     return data
 
 
-def get_sitemap(link: Link) -> typing.Optional[typing.List[File]]:  # pylint: disable=inconsistent-return-statements
+def get_sitemaps(link: Link) -> typing.Optional[typing.List[File]]:  # pylint: disable=inconsistent-return-statements
     """Read sitemaps.
 
     Args:
@@ -149,7 +160,7 @@ def get_sitemap(link: Link) -> typing.Optional[typing.List[File]]:  # pylint: di
     """
     path_list = glob.glob(os.path.join(link.base, 'sitemap_*.xml'))
     if not path_list:
-        return
+        return None
 
     data_list = list()
     for path in path_list:
@@ -184,11 +195,11 @@ def get_hosts(link: Link) -> typing.Optional[File]:  # pylint: disable=inconsist
 
     """
     if link.proxy != 'i2p':
-        return
+        return None
 
     path = os.path.join(link.base, 'hosts.txt')
     if not os.path.isfile(path):
-        return
+        return None
     with open(path, 'rb') as file:
         content = file.read()
     data = dict(
@@ -247,7 +258,7 @@ def submit(api: str, domain: Domain, data: typing.Dict[str, typing.Any]):
                 if response.ok:
                     return
             except requests.RequestException as error:
-                warning = warnings.formatwarning(error, APIRequestFailed, __file__, 150,
+                warning = warnings.formatwarning(str(error), APIRequestFailed, __file__, 252,
                                                  f'[{domain.upper()}] response = requests.post(api, json=data)')
                 print(render_error(warning, stem.util.term.Color.YELLOW), end='', file=sys.stderr)  # pylint: disable=no-member
     save_submit(domain, data)
@@ -269,7 +280,9 @@ def submit_new_host(time: typing.Datetime, link: Link, partial: bool = False):
     If :data:`~darc.submit.API_NEW_HOST` is :data:`None`, the data for submission
     will directly be save through :func:`~darc.submit.save_submit`.
 
-    The data submitted should have following format::
+    The data submitted should have following format:
+
+    .. code-block:: jsonc
 
         {
             // partial flag - true / false
@@ -307,7 +320,7 @@ def submit_new_host(time: typing.Datetime, link: Link, partial: bool = False):
             "Sitemaps": [
                 {
                     // path of the file, relative path (to data root path ``PATH_DATA``) in container
-                    //   - <proxy>/<scheme>/<host>/sitemap_<name>.txt
+                    //   - <proxy>/<scheme>/<host>/sitemap_<name>.xml
                     "path": ...,
                     // content of the file (**base64** encoded)
                     "data": ...,
@@ -330,7 +343,7 @@ def submit_new_host(time: typing.Datetime, link: Link, partial: bool = False):
         * :func:`darc.submit.save_submit`
         * :func:`darc.submit.get_metadata`
         * :func:`darc.submit.get_robots`
-        * :func:`darc.submit.get_sitemap`
+        * :func:`darc.submit.get_sitemaps`
         * :func:`darc.submit.get_hosts`
 
     """
@@ -338,41 +351,39 @@ def submit_new_host(time: typing.Datetime, link: Link, partial: bool = False):
     ts = time.isoformat()
 
     robots = get_robots(link)
-    sitemap = get_sitemap(link)
+    sitemaps = get_sitemaps(link)
     hosts = get_hosts(link)
 
     if SAVE_DB:
         try:
-            model, _ = HostnameModel.get_or_create(hostname=link.host, defaults=dict(
-                proxy=HostnameModel.Proxy[link.proxy.upper()],
+            model, _ = _db_operation(HostnameModel.get_or_create, hostname=link.host, defaults=dict(
+                proxy=Proxy[link.proxy.upper()],
                 discovery=time,
                 last_seen=time,
                 alive=False,
-                since=time,
+                since=EPOCH,
             ))
 
             if robots is not None:
-                RobotsModel.create(
-                    host=model,
-                    timestamp=time,
-                    document=base64.b64decode(robots['data']).decode(),
-                )
+                _db_operation(RobotsModel.create,
+                              host=model,
+                              timestamp=time,
+                              document=base64.b64decode(robots['data']).decode())
 
-            if sitemap is not None:
-                SitemapModel.create(
-                    host=model,
-                    timestamp=time,
-                    document=base64.b64decode(sitemap['data']).decode(),
-                )
+            if sitemaps is not None:
+                for sitemap in sitemaps:
+                    _db_operation(SitemapModel.create,
+                                  host=model,
+                                  timestamp=time,
+                                  document=base64.b64decode(sitemap['data']).decode())
 
             if hosts is not None:
-                HostsModel.create(
-                    host=model,
-                    timestamp=time,
-                    document=base64.b64decode(hosts['data']).decode(),
-                )
+                _db_operation(HostsModel.create,
+                              host=model,
+                              timestamp=time,
+                              document=base64.b64decode(hosts['data']).decode())
         except Exception as error:
-            warning = warnings.formatwarning(error, DatabaseOperaionFailed, __file__, 347,
+            warning = warnings.formatwarning(str(error), DatabaseOperaionFailed, __file__, 354,
                                              'submit_new_host(...)')
             print(render_error(warning, stem.util.term.Color.YELLOW), end='', file=sys.stderr)  # pylint: disable=no-member
 
@@ -382,7 +393,7 @@ def submit_new_host(time: typing.Datetime, link: Link, partial: bool = False):
         'Timestamp': ts,
         'URL': link.host,
         'Robots': robots,
-        'Sitemaps': sitemap,
+        'Sitemaps': sitemaps,
         'Hosts': hosts,
     }
 
@@ -422,7 +433,9 @@ def submit_requests(time: typing.Datetime, link: Link,
     If :data:`~darc.submit.API_REQUESTS` is :data:`None`, the data for submission
     will directly be save through :func:`~darc.submit.save_submit`.
 
-    The data submitted should have following format::
+    The data submitted should have following format:
+
+    .. code-block:: jsonc
 
         {
             // metadata of URL
@@ -468,7 +481,7 @@ def submit_requests(time: typing.Datetime, link: Link,
             "Response": {
                 ...
             },
-            // Content type
+            // content type
             "Content-Type": ...,
             // requested file (if not exists, then ``null``)
             "Document": {
@@ -498,63 +511,63 @@ def submit_requests(time: typing.Datetime, link: Link,
     """
     if SAVE_DB:
         try:
-            model, _ = HostnameModel.get_or_create(hostname=link.host, defaults=dict(
-                proxy=HostnameModel.Proxy[link.proxy.upper()],
+            model, _ = _db_operation(HostnameModel.get_or_create, hostname=link.host, defaults=dict(
+                proxy=Proxy[link.proxy.upper()],
                 discovery=time,
                 last_seen=time,
                 alive=False,
-                since=time,
+                since=EPOCH,
             ))
-            if not model.alive:
+            if not model.alive and response.ok:
                 model.alive = True
                 model.since = time
-                model.save()
+            model.last_seen = time
+            _db_operation(model.save)
 
-            url, _ = URLModel.get_or_create(hash=link.name, defaults=dict(
+            url, _ = _db_operation(URLModel.get_or_create, hash=link.name, defaults=dict(
                 url=link.url,
                 hostname=model,
-                proxy=URLModel.Proxy[link.proxy.upper()],
+                proxy=Proxy[link.proxy.upper()],
                 discovery=time,
                 last_seen=time,
                 alive=False,
-                since=time,
+                since=EPOCH,
             ))
-            if not url.alive and response.ok:
+            if not model.alive and response.ok:
                 url.alive = True
                 url.since = time
-                url.save()
+            url.last_seen = time
+            _db_operation(url.save)
 
-            model = RequestsModel.create(
-                url=url,
-                timestamp=time,
-                method=response.request.method,
-                document=content,
-                mime_type=mime_type,
-                is_html=html,
-                status_code=response.status_code,
-                reason=response.reason,
-                cookies=response.cookies.get_dict(),
-                session=response.cookies.get_dict(),
-                request=dict(response.request.headers),
-                response=dict(response.headers),
-            )
+            model = _db_operation(RequestsModel.create,
+                                  url=url,
+                                  timestamp=time,
+                                  method=response.request.method,
+                                  document=content,
+                                  mime_type=mime_type,
+                                  is_html=html,
+                                  status_code=response.status_code,
+                                  reason=response.reason,
+                                  cookies=response.cookies.get_dict(),
+                                  session=response.cookies.get_dict(),
+                                  request=dict(response.request.headers),
+                                  response=dict(response.headers))
 
             for index, history in enumerate(response.history):
-                RequestsHistoryModel.create(
-                    index=index,
-                    model=model,
-                    url=history.url,
-                    timestamp=time,
-                    method=history.request.method,
-                    document=history.content,
-                    status_code=history.status_code,
-                    reason=history.reason,
-                    cookies=history.cookies.get_dict(),
-                    request=dict(history.request.headers),
-                    response=dict(history.headers),
-                )
+                _db_operation(RequestsHistoryModel.create,
+                              index=index,
+                              model=model,
+                              url=history.url,
+                              timestamp=time,
+                              method=history.request.method,
+                              document=history.content,
+                              status_code=history.status_code,
+                              reason=history.reason,
+                              cookies=history.cookies.get_dict(),
+                              request=dict(history.request.headers),
+                              response=dict(history.headers))
         except Exception as error:
-            warning = warnings.formatwarning(error, DatabaseOperaionFailed, __file__, 502,
+            warning = warnings.formatwarning(str(error), DatabaseOperaionFailed, __file__, 509,
                                              'submit_requests(...)')
             print(render_error(warning, stem.util.term.Color.YELLOW), end='', file=sys.stderr)  # pylint: disable=no-member
 
@@ -631,7 +644,9 @@ def submit_selenium(time: typing.Datetime, link: Link,
         :mod:`requests` is HTML, status code not between ``400`` and ``600``, and
         HTML data not empty.
 
-    The data submitted should have following format::
+    The data submitted should have following format:
+
+    .. code-block:: jsonc
 
         {
             // metadata of URL
@@ -685,32 +700,41 @@ def submit_selenium(time: typing.Datetime, link: Link,
     """
     if SAVE_DB:
         try:
-            model, _ = HostnameModel.get_or_create(hostname=link.host, defaults=dict(
-                proxy=HostnameModel.Proxy[link.proxy.upper()],
+            model, _ = _db_operation(HostnameModel.get_or_create, hostname=link.host, defaults=dict(
+                proxy=Proxy[link.proxy.upper()],
                 discovery=time,
                 last_seen=time,
-                alive=False,
+                alive=True,
                 since=time,
             ))
+            if not model.alive:
+                model.alive = True
+                model.since = time
+            model.last_seen = time
+            _db_operation(model.save)
 
-            url, _ = URLModel.get_or_create(hash=link.name, defaults=dict(
+            url, _ = _db_operation(URLModel.get_or_create, hash=link.name, defaults=dict(
                 url=link.url,
                 hostname=model,
-                proxy=URLModel.Proxy[link.proxy.upper()],
+                proxy=Proxy[link.proxy.upper()],
                 discovery=time,
                 last_seen=time,
-                alive=False,
+                alive=True,
                 since=time,
             ))
+            if not url.alive:
+                url.alive = True
+                url.since = time
+            url.last_seen = time
+            _db_operation(url.save)
 
-            SeleniumModel.create(
-                url=url,
-                timestamp=time,
-                document=html,
-                screenshot=base64.b64decode(screenshot),
-            )
+            _db_operation(SeleniumModel.create,
+                          url=url,
+                          timestamp=time,
+                          document=html,
+                          screenshot=base64.b64decode(screenshot) if screenshot else None)
         except Exception as error:
-            warning = warnings.formatwarning(error, DatabaseOperaionFailed, __file__, 681,
+            warning = warnings.formatwarning(str(error), DatabaseOperaionFailed, __file__, 696,
                                              'submit_selenium(...)')
             print(render_error(warning, stem.util.term.Color.YELLOW), end='', file=sys.stderr)  # pylint: disable=no-member
 
@@ -748,4 +772,4 @@ def submit_selenium(time: typing.Datetime, link: Link,
         return
 
     # submit data
-    submit(API_REQUESTS, 'selenium', data)
+    submit(API_SELENIUM, 'selenium', data)
